/*
* $Log: lords.h,v $
* Revision 1.6  2004/01/30 22:48:50  Default
* Dynamic memory stuff removed - replaced by static ewram assignment
*
* Revision 1.5  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.4  2004/01/25 21:24:48  richq
* Update stats at night, recalculate graphics when horse lost/found
*
* Revision 1.3  2004/01/22 20:24:41  richq
* Removed think, choose, etc code and placed it in seperate modules.
*
* Revision 1.2  2004/01/22 20:03:52  richq
* The Think Screen code is added.
*
* Revision 1.1  2004/01/21 19:20:04  Default
* Stuff to do with lords. Looking, thinking, moving, etc.
*
*/

#ifndef LORDS_H
#define LORDS_H

#include "gba.h"
#include "location.h"

#define LUXOR 0
#define MORKIN 1
#define FARFLAME 31

#define LOOK_SCREEN 1
#define THINK_SCREEN 2
#define CHOOSE_SCREEN 3
#define SELECT_SCREEN 4
#define MENU_SCREEN 5

typedef struct {
  location_t location;
  u8 time;                //BYTE 03 - time of day
  u8 firstname;           //BYTE 04 - first name token
  u8 title;               //BYTE 05 
  u8 moonring;            //BYTE
  u8 graphic;             //BYTE 07 
  u8 riders;              //BYTE 08 - total riders / 5
  u8 riders_energy;       //BYTE 09 - riders energy
  u8 warriors;            //BYTE 10 - total warriors / 5
  u8 warriors_energy;     //BYTE 11 - warriors energy
  u8 battle_area;         //BYTE 12 - area currently in battle in
  u8 riders_lost;         //BYTE 13 - number of riders lost in battle
  u8 warriors_lost;       //BYTE 14 - number of warriors lost in battle
  u8 killed;              //BYTE 15 - number of the enemy killed by the character in battle
  u8 riders_killed;       //BYTE 16 - enemy killed by riders
  u8 warriors_killed;     //BYTE 17 - enemy killed by warriors
  u8 battle_status;       //BYTE 18 - 0xff no batle, 0 - battle continues, ?? - who won
  u8 alive;               //BYTE 19 - 0 or 1
  u8 energy;              //BYTE 20 - current energy
  u8 strength;            //BYTE 21 - current strength
  u8 cowardess;           //BYTE 22 
  u8 recruiting_key;      //BYTE 23 the character who is doing the recruiting, recruiting_key is
  u8 recruited_by;        //BYTE 24 anded with the recruited_by from the character he wished to recruit and if it is not zero
                          //        then recruiting can take place.
  u8 courage;             //BYTE 25 
  u8 unused;              //BYTE 26 
  u8 hidden;              //BYTE 27 - 0 or 1
  u8 race;                //BYTE 28 - see race table
  u8 horse;               //BYTE 29 - 0 or 1
  u8 object;              //BYTE 30 - object carrying
  u8 deathby;             //BYTE 31 , 0 - in battle, otherwise the value is the object that killed him
} lord_t;


// currently selected screen
extern int g_lordScreen;
extern int g_gameover;

// 6c00, 32 entries
extern lord_t * const g_lordTable;

extern lord_t * g_selectedLord;
extern u8 g_selectedLordID;

void init_lordtable(void);

void setSelectedLord(int id);
//void saveSelectedLord(void);

void updateCurrentInfo(void);

void calculateCourage(void);

// get the id of a new ally in the current square
int getRecruitableID(void);

// increase stats by rest period amount + increment
void updateStats(int increment);
// update the lords for the dawn 
void resetLordStats(void);


void calculateGFX(void);

void toggleControl(int toggleFlag);

#endif

