/*
* $Log: objects.c,v $
* Revision 1.8  2004/02/18 20:00:24  richq
* Fixed a potential out-of-OAM bug that could have occurred when several
* different types of lord/creatures accumulated in a single location.
*
* Revision 1.7  2004/01/31 15:33:30  Default
* Palette stuff moved to gfx.c
*
* Revision 1.6  2004/01/30 22:57:03  Default
* X positions of objects altered a bit to fit gba screen better
*
* Revision 1.5  2004/01/27 20:19:13  richq
* The temp lord object has changed - it's now a pointer to the actual
* table entry and gets rid of the need for all the copying
* from temp to the table and back again.
*
* Revision 1.4  2004/01/25 21:27:00  richq
* Small adjustment for transparentness, first row and edges made transparent
* the rest is solid.
*
* Revision 1.3  2004/01/22 22:36:41  richq
* Objects palette now distinct to shield. Night time palette trick added.
* Y positions uses graphic size, so line up with the bottom row okay.
*
* Revision 1.2  2004/01/22 20:02:22  richq
* Now draws more objects in front of us. Objects descriptions for
* on the thinking screen added. Bit of optimisation of vram added too:
* doesn't unpack the same gaphics several times, but references
* repeated character gfx in OAM.
*                                                     
* Revision 1.1  2004/01/21 19:23:09  Default                              
* Draws the "objects" - character graphics. These are drawn to GBA sprites
* and makes use of a few routines in gfx.c.                               
*                                                                         
*/

#include "gba.h"

#include "gfx.h"
#include "sprite.h"
#include "objects.h"
#include "objectdata.h"
#include "location.h"
#include "army.h"
#include "lords.h"
#include "screenmode.h"
#include "tokens.h"
#include "tokendata.h"
#include "look.h"

#define OBJECT_PAL_OFFSET 17

// the object at a location
// this is initially 0x10 when we move to a square
// unless there is a nasty in the square, gets set to the nasty id
// when we seek, if we find nothing, set to 0. Otherwise set to seek object id.
int g_tmpObject;
// used for guidance, the random guidance advice
// also used when we slay things or find wild horses
int g_objectKey;

#define objectGfxData _binary_objectgfx_bin_start
extern u8 objectGfxData[];

const u8 objGfxType[] = {
   GFX_WOLF,
   GFX_DRAGON,
   GFX_ICE_TROLL,
   GFX_SKULKRIN,
   GFX_WILD_HORSE
};


const u8 objPosition[] = {
   0x0b, //1
   0x13, //2
   0x0f, //3
   0x03, //4
   0x07, //5
   0x17, //6
   0xff, //7
   0x1b, //8
};


int g_useNightimeObjectPal;

void drawGFX(int id, int x, int y, int sprite) {
  
  OBJPaletteMem[7+OBJECT_PAL_OFFSET] = BGPaletteMem[18];
  
  const objectgfx_t * theGFX = objectGFXTable[id];
  int i, j;
  int mapIndex = 0;
  int colourIndex = 0;
  int tile = 0;
  int colour0, colour1;
  u8 tileBuffer[8*8];
  
  gfxTable = objectGfxData;
  int paper, ink;
  for (j = 0; j < theGFX->height; j++) {
    for (i = 0; i < theGFX->width; i++) {
      tile = theGFX->tileMap[mapIndex];
      if (tile != 0) {
        colour0 = theGFX->colourMap[colourIndex]&0x7;   // experimental...
        colour1 = (theGFX->colourMap[colourIndex]>>3)&0x7;   // experimental...
        
        if (g_useNightimeObjectPal) {
          paper = 0;
          ink = 1+OBJECT_PAL_OFFSET;
        }
        else {
         paper = (colour1==7 && (j==0 || i==0 || i==theGFX->width-1) )?0
                 : colour1+OBJECT_PAL_OFFSET;
         ink = colour0+OBJECT_PAL_OFFSET;
        }
        createTile( tileBuffer,
                    tile, 
                    ink, //colour0+OBJECT_PAL_OFFSET,
                    paper //(colour1==7)?0:colour1+OBJECT_PAL_OFFSET
                   );
        drawImage(tileBuffer,i, j, OBJECT_SPRITE+sprite);
        
      }
      mapIndex++;
      colourIndex++;
    }
  }
  
  init_sprite64(OBJECT_SPRITE+sprite, 0);
//  move_sprite(OBJECT_SPRITE+sprite, x*8, y*8);
  MoveSprite(&sprites[OBJECT_SPRITE+sprite], x*8, y*8);
}

// cache the previous gfx type to reuse it
int previousType, reusedOAM;
void drawObject(int i, int y, int *amount){
  if (*amount < MAX_OBJECTS-1) {
    int x = objPosition[*amount];
    if (previousType == i) {
      reusedOAM = 1;
      int sprite = OBJECT_SPRITE + (*amount);
      init_sprite64(sprite, 0);
      sprites[sprite].attribute2 = sprites[sprite-1].attribute2;
      MoveSprite(&sprites[sprite], x*8, y*8);
    }
    else {
      // fixes a possible bug if a certain combo of people get together 
      // in one spot - any non repeating combo that circumvents caching
      if ( (*amount < MAX_OBJECTS-2) || 
        (reusedOAM && *amount == MAX_OBJECTS-2) ) 
      {
        previousType = i;
        drawGFX(i, x, y, *amount);
      }
      
    }
    (*amount)++;
  }
}


// 655d
void drawLordsHere(int * amount) {
  int i;
  for (i = 0; i < g_lordsHere; i++) {
    setSelectedLord(g_charTable[i]); // this line is CALL fd04!
    drawObject(g_selectedLord->graphic, 
               0x14-objectGFXTable[g_selectedLord->graphic]->height, 
               amount);
  }
  
}


/*
drawInFront
 draw whatever is infront of us. 
 Aftre this, will need to call updateCurrentInfo to get back current details
 based on 6510.
*/
void drawInFront(void) {
  // use g_inFront for the infront square
  // set day and night attribute based on g_selectLord.time
  // not yet done...
  g_useNightimeObjectPal = !g_selectedLord->time;
  // drawing in the day? Use the bright palette
  set_object_palette(1);
  
  // find details of square immediately ahead of us
  workOutLocationDetails(&g_inFront);
  // set the flag to say if we can move forward or not
  setForwardsBlocked();
  
  // amount of things drawn:
  int amount = 0;
  previousType = -1;
  reusedOAM = 0;
  if (g_lordsHere) {
    drawLordsHere(&amount);
  }

  if (g_doomdarkRiders) {
    int i;
    for (i = 0; i < 8; i++) {
      drawObject(GFX_DOOM_RIDER, 
                 0x14-objectGFXTable[GFX_DOOM_RIDER]->height, //0xD, 
                 &amount);
    } 
  }

  if (g_doomdarkWarriors) {
    int i;
    for (i = 0; i < 8; i++) {
      drawObject(GFX_DOOM_WARRIOR, 
                 0x14-objectGFXTable[GFX_DOOM_WARRIOR]->height, //0xD, 
                 &amount);
    } 
  }
  
  
  //int y = 0xD;   // was base.. 0x17. Now top 14*8 = 112
  if (g_locdat.object != 0 && g_locdat.object < 6) {    
    int obj = g_locdat.object-1;
    if (objGfxType[obj] != GFX_DRAGON) {
      drawObject(objGfxType[obj], 
                 0x14-objectGFXTable[objGfxType[obj]]->height,
                 &amount);
      drawObject(objGfxType[obj], 
                 0x14-objectGFXTable[objGfxType[obj]]->height,
                 &amount);
    }
    drawObject(objGfxType[obj], 
                 0x14-objectGFXTable[objGfxType[obj]]->height,
                 &amount);
    drawObject(objGfxType[obj], 
                 0x14-objectGFXTable[objGfxType[obj]]->height,
                 &amount);
  }
  
  CopyOAM();
  g_useNightimeObjectPal = 0;
}

void set_object_palette(int bright) {
  int i;
  for (i = 0 ; i < 8; i++) {
    if (bright)
      OBJPaletteMem[i+OBJECT_PAL_OFFSET] = brightPal[i];
    else
      OBJPaletteMem[i+OBJECT_PAL_OFFSET] = nonBrightPal[i];
  }
  
}


// sets g_tmpObject, the object at the lords location
// originally mixed in with the courage code at 65dc
void setObjectHere(void) {
  if (g_locdat.object == 0 || g_locdat.object >= 6) {
    g_tmpObject = 0x10;
  } else {
    g_tmpObject = g_locdat.object;
  }
  g_objectKey = 0;
}

void heHasFound(int found) {
  //ecb1
  addToTokenList(hehasfound);
  addToTokenList(objectDescriptions[found]);
  if (found == OBJECT_GUIDANCE) {
    // eccc
    addLiteralToken('%');   // '%' = opening "
    if (g_objectKey < 4) {
      // "so-and-so can destroy the ice crown"
      addToTokenList(objectDescriptions[g_objectKey+0x10]);
      addToTokenList(canDestroyCrown);
    }
    else {
      // "looking for blah, you must seek bloo"
      setSelectedLord(g_objectKey);
      locdat_t loc;
      loc.x = g_selectedLord->location.x;
      loc.y = g_selectedLord->location.y;
      fillLocationData(&loc);
      addToTokenList(lookingFor);
      createLordTitle(g_selectedLord->title, g_selectedLord->firstname);
      addLiteralToken(',');
      addToTokenList(youMustSeek);
      addDomainTokens(&loc);
    }
    addLiteralToken('.');
    addLiteralToken('&');  // '&' = closing "
    addNewlineToken();
  } else {
    addLiteralToken('.');
  }
}


//ed87
void nastiesAbroad(int nastyid) {
  if (nastyid == OBJECT_WILD_HORSES) {
    // eda4
    if (g_objectKey) {
      // do found
      heHasFound(nastyid);
      return;
    } else {
      // this next line replaces e49a!
      addToTokenList(objectDescriptions[nastyid]);
      addToTokenList(areAbroad);
    }
  } else {
    // nasty != OBJECT_WILD_HORSES
    if (g_objectKey) {
      // killed it.. edb7
      addToTokenList(heslewthe);
      addToTokenList(objectDescriptions[nastyid]);
      
      if (g_selectedLord->object < OBJECT_ICE_CROWN &&
          g_selectedLord->object > OBJECT_CUP) 
      {
        // what with?
        addSingleToken(0xe3); // with
        addToTokenList(objectDescriptions[g_selectedLord->object]);
      }
    } else {
      // fight or hide!
      addToTokenList(objectDescriptions[nastyid]);
      addToTokenList(areAbroadNasty);
    }
    
  }
  addLiteralToken('.');
}


/*part of the think screen code from eca0*/
void thinkAboutObjects(void) {
  if (g_tmpObject == OBJECT_NOTHING || (g_tmpObject < OBJECT_EMPTY && g_tmpObject >= OBJECT_SHELTER)) {
    // seeked 
    heHasFound(g_tmpObject);
  } else if (g_tmpObject < OBJECT_SHELTER) {
    // nasties about ed87
    nastiesAbroad(g_tmpObject);
  } 
  // else if (g_tmpObject >= OBJECT_EMPTY) {
  // not seeked, no nasties, so do nothing
  //return;
  //}
}


void clear_objects(void) {
  int i;
  for (i = 0; i < MAX_OBJECTS; i++) {
    remove_sprite(OBJECT_SPRITE + i);
  }
  CopyOAM();
  
  // clear vram too
  for (i = 0; i < MAX_OBJECTS-2; i++) {
    clearSpriteRAM(OBJECT_SPRITE + i);
  }
  
}
