/*
* $Log: shield.c,v $
* Revision 1.5  2004/01/31 15:36:28  Default
* Shield palette is now brightPal in gfx.c
*
* Revision 1.4  2004/01/25 21:28:56  richq
* Remove shield added, now only redraws shield when needed. Shield
* made to go behind background.
*
* Revision 1.3  2004/01/22 20:28:12  richq
* New text colour... probably shouldn't be in here!
*
* Revision 1.2  2004/01/22 19:57:03  richq
* Shield sprite priority in an attempt to control where it goes (in front
* or behind the backdrop). colours loaded to BG palette for the text.
*                                 
* Revision 1.1  2004/01/21 19:28:34  Default         
* Routines and data for drawing the character shields
*                                                    
*/

#include "gba.h"
#include "dma.h"
#include "shield.h"
#include "screenmode.h"
#include "gfx.h"


#define SHIELD_X 190
#define SHIELD_Y 2

// first some data:

// a table of 256 bytes
// each entry up to 8 bytes big
// first byte = main colour
// next byte:
//        lower 3 bits = pattern
//        upper bits = colour adjuster
// next byte:
//        low 3 bits = x offset
//        rest / 8  = y offset
#define shieldMainTable _binary_shieldmain_bin_start
extern const u8 shieldMainTable[];// = 0xdea8;

// 8*8 speccy format tiles. contains 36 tiles (8*37 = 296 bytes)
#define shieldGfxTable _binary_shieldgfx_bin_start
extern const u8 shieldGfxTable[]; //  = 0xdd1b;

const u8 pattern0[] = {
   0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
   0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
   0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
   0x02, 0x01, 0x01, 0x01, 0x01, 0x05,
   0x03, 0x01, 0x01, 0x01, 0x01, 0x06,
   0x00, 0x04, 0x01, 0x01, 0x07, 0x00,
   0x00, 0x00, 0x04, 0x07, 0x00, 0x00
};

const u8 pattern1[] = {
   0x08, 0x09,
   0x0a, 0x0b,
   0x0c, 0x0d
};
const u8 pattern2[] = {
   0x0e, 0x10,
   0x0f, 0x11
};
  
const u8 pattern3[] = {
   0x12,
   0x13
};
const u8 pattern4[] = {
   0x14, 0x15, 0x16
};
const u8 pattern5[] = {
   0x17, 0x18, 0x19, 0x1a,
   0x00, 0x1b, 0x1c, 0x00
};
const u8 pattern6[] = {
   0x1d, 0x1e, 0x1f, 0x20, 
   0x21, 0x22, 0x23, 0x24
}; 

typedef struct {
  const u8 * const data;
  u8 width;
  u8 height;
} pattern_t;

  

const pattern_t patterns[] = {
  {pattern0, 6, 7},
  {pattern1, 2, 3},
  {pattern2, 2, 2},
  {pattern3, 1, 2},
  {pattern4, 3, 1},
  {pattern5, 4, 2},
  {pattern6, 4, 2}
};

int g_lastShield;
// shields are sprites. they use the bright palette
void init_shield_pal(void) {
  int i;
  for (i = 0 ; i < 8; i++) {
    OBJPaletteMem[i+1] = brightPal[i];
  }
  g_lastShield =-1;
}


void createPattern(int token, int xoffset, int yoffset, 
                    int col0, int colOffset) {
  
  u8 tileBuffer[8*8];
  gfxTable = shieldGfxTable;
  const pattern_t * p = &patterns[token];
  int index = 0;
  int x, y;
  for (y = 0; y < p->height; y++) {
    for (x = 0; x < p->width; x++) {
      int tile = p->data[index++];
      if (tile != 0) {
        // draw the thing here.
        createTile( tileBuffer,
                    tile, 
                    col0+1,
                    //colOffset
                    (colOffset == 0 ? 0:colOffset+1)
                   );
        // copy the tilebuffer into vram:
        drawImage(tileBuffer,(x+xoffset), (y+yoffset), SHIELD_SPRITE);
      }
    }
  }
  
}


void remove_shield(void) {
  remove_sprite(SHIELD_SPRITE);
  clearSpriteRAM(SHIELD_SPRITE);
  g_lastShield = -1;
}

void drawShield(int shield) {
  if (shield > 31)
    return;
  if (shield == g_lastShield)
    return;
  // faa8 
  //remove_sprite(SHIELD_SPRITE);
  int colour0 = shieldMainTable[shield*8];
  int colourOff = 0;
  int xoffset = 0;
  int yoffset = 0;
  u8 byte1, byte2;
  int token = 0;
  int index = 1;
  for (;;) {
    createPattern(token, xoffset, yoffset, colour0, colourOff);
    
    byte1 = shieldMainTable [index + shield*8];
    if (byte1 == 0)
      break;
    index++;
    byte2 = shieldMainTable[index + shield*8];
    index++;
    
    token = byte1&7;
    colourOff = (byte1)/8;
    xoffset = byte2&7;
    yoffset = (byte2)/8;
  }
  init_sprite64(SHIELD_SPRITE, 2);
  move_sprite(SHIELD_SPRITE, SHIELD_X, SHIELD_Y);
  g_lastShield = shield;
}

